/******************************************************************************
 * Product: Compiere ERP & CRM Smart Business Solution                        *
 * Copyright (C) 1999-2007 ComPiere, Inc. All Rights Reserved.                *
 * This program is free software, you can redistribute it and/or modify it    *
 * under the terms version 2 of the GNU General Public License as published   *
 * by the Free Software Foundation. This program is distributed in the hope   *
 * that it will be useful, but WITHOUT ANY WARRANTY, without even the implied *
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.           *
 * See the GNU General Public License for more details.                       *
 * You should have received a copy of the GNU General Public License along    *
 * with this program, if not, write to the Free Software Foundation, Inc.,    *
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.                     *
 * For the text or an alternative of this public license, you may reach us    *
 * ComPiere, Inc., 3600 Bridge Parkway #102, Redwood City, CA 94065, USA      *
 * or via info@compiere.org or http://www.compiere.org/license.html           *
 *****************************************************************************/
package org.compiere.model;

import java.math.*;
import java.sql.*;
import java.util.Calendar;
import java.util.logging.*;

import org.compiere.util.*;

/**
 *	Cash Book Journal Callouts
 *	
 *  @author Jorg Janke
 *  @version $Id: CalloutCashJournal.java,v 1.2 2010/04/17 05:01:27 jrmt Exp $
 *  @deprecated
 */
public class CalloutCashJournal extends CalloutEngine
{
	/**
	 *  Cash Journal Line Invoice.
	 *  when Invoice selected
	 *  - set C_Currency, DiscountAnt, Amount, WriteOffAmt
	 *	@param ctx context
	 *	@param WindowNo window no
	 *	@param mTab tab
	 *	@param mField field
	 *	@param value value
	 *	@return null or error message
	 */
	public String cashbook (Ctx ctx, int WindowNo, GridTab mTab, GridField mField, Object value)
	{
		if (isCalloutActive())		//	assuming it is resetting value
			return "";
		setCalloutActive(true);

		Integer C_CashBook_ID = (Integer)mTab.getValue("C_CashBook_ID");
		if (C_CashBook_ID == null || C_CashBook_ID.intValue() == 0)
		{
			mTab.setValue("C_Currency_ID", null);
			setCalloutActive(false);
			return "";
		}

		Timestamp statementDate = (Timestamp)mTab.getValue("StatementDate");
		if( statementDate == null )
			statementDate = new Timestamp(Calendar.getInstance().getTimeInMillis());
		Integer C_Cash_ID = (Integer)mTab.getValue("C_Cash_ID");
		if( C_Cash_ID == null )
			C_Cash_ID = 0;
		BigDecimal beginningBalance = MCash.getBalance(C_CashBook_ID, statementDate, C_Cash_ID);
		
		mTab.setValue("BeginningBalance", beginningBalance);

		setCalloutActive(false);
		return "";
	}	//	CashJournal_CashBook

	/**
	 *  Cash Journal Line Invoice.
	 *  when Invoice selected
	 *  - set C_Currency, DiscountAnt, Amount, WriteOffAmt
	 *	@param ctx context
	 *	@param WindowNo window no
	 *	@param mTab tab
	 *	@param mField field
	 *	@param value value
	 *	@return null or error message
	 */
	public String invoice (Ctx ctx, int WindowNo, GridTab mTab, GridField mField, Object value)
	{
		if (isCalloutActive())		//	assuming it is resetting value
			return "";
		setCalloutActive(true);

		Integer C_Invoice_ID = (Integer)value;
		if (C_Invoice_ID == null || C_Invoice_ID.intValue() == 0)
		{
			mTab.setValue("C_Currency_ID", null);
			setCalloutActive(false);
			return "";
		}

		//  Date
		Timestamp ts = new Timestamp(ctx.getContextAsTime(WindowNo, "DateAcct"));     //  from C_Cash
		String sql = "SELECT C_BPartner_ID, C_Currency_ID,"		//	1..2
			+ "invoiceOpen(C_Invoice_ID, 0), IsSOTrx, "			//	3..4
			+ "paymentTermDiscount(invoiceOpen(C_Invoice_ID, 0),C_Currency_ID,C_PaymentTerm_ID,DateInvoiced,?) "
			+ "FROM C_Invoice WHERE C_Invoice_ID=?";
		try
		{
			PreparedStatement pstmt = DB.prepareStatement(sql, null);
			pstmt.setTimestamp(1, ts);
			pstmt.setInt(2, C_Invoice_ID.intValue());
			ResultSet rs = pstmt.executeQuery();
			if (rs.next())
			{
				mTab.setValue("C_BPartner_ID", new Integer(rs.getInt(1)));
				mTab.setValue("C_Currency_ID", new Integer(rs.getInt(2)));
				BigDecimal PayAmt = rs.getBigDecimal(3);
				BigDecimal DiscountAmt = rs.getBigDecimal(5);
				boolean isSOTrx = "Y".equals(rs.getString(4));
				if (!isSOTrx)
				{
					PayAmt = PayAmt.negate();
					DiscountAmt = DiscountAmt.negate();
				}
				//
				mTab.setValue("Amount", PayAmt.subtract(DiscountAmt));
				mTab.setValue("DiscountAmt", DiscountAmt);
				mTab.setValue("WriteOffAmt", Env.ZERO);
				ctx.setContext( WindowNo, "InvTotalAmt", PayAmt.toString());
			}
			rs.close();
			pstmt.close();
		}
		catch (SQLException e)
		{
			log.log(Level.SEVERE, "invoice", e);
			setCalloutActive(false);
			return e.getLocalizedMessage();
		}
		setCalloutActive(false);
		return "";
	}	//	CashJournal_Invoice

	
	/**
	 *  Cash Journal Line Invoice Amounts.
	 *  when DiscountAnt, Amount, WriteOffAmt change
	 *  making sure that add up to InvTotalAmt (created by CashJournal_Invoice)
	 *	@param ctx context
	 *	@param WindowNo window no
	 *	@param mTab tab
	 *	@param mField field
	 *	@param value value
	 *	@return null or error message
	 */
	public String amounts (Ctx ctx, int WindowNo, GridTab mTab, GridField mField, Object value)
	{
		//  Needs to be Invoice
		if (isCalloutActive() || !"I".equals(mTab.getValue("CashType")))
			return "";
		//  Check, if InvTotalAmt exists
		String total = ctx.getContext( WindowNo, "InvTotalAmt");
		if (total == null || total.length() == 0)
			return "";
		BigDecimal InvTotalAmt = new BigDecimal(total);
		setCalloutActive(true);

		BigDecimal PayAmt = (BigDecimal)mTab.getValue("Amount");
		BigDecimal DiscountAmt = (BigDecimal)mTab.getValue("DiscountAmt");
		BigDecimal WriteOffAmt = (BigDecimal)mTab.getValue("WriteOffAmt");
		String colName = mField.getColumnName();
		log.fine(colName + " - Invoice=" + InvTotalAmt
			+ " - Amount=" + PayAmt + ", Discount=" + DiscountAmt + ", WriteOff=" + WriteOffAmt);

		//  Amount - calculate write off
		if (colName.equals("Amount"))
		{
			WriteOffAmt = InvTotalAmt.subtract(PayAmt).subtract(DiscountAmt);
			mTab.setValue("WriteOffAmt", WriteOffAmt);
		}
		else    //  calculate PayAmt
		{
			PayAmt = InvTotalAmt.subtract(DiscountAmt).subtract(WriteOffAmt);
			mTab.setValue("Amount", PayAmt);
		}

		setCalloutActive(false);
		
		return "";
	}	//	amounts

	// Jirimuto modified for adding for charge tax calculation --start 2010/04/14
	/**
	 * @param ctx
	 * @param WindowNo
	 * @param mTab
	 * @param mField
	 * @param value
	 * @return
	 */
	public String charge (Ctx ctx, int WindowNo, GridTab mTab, GridField mField, Object value)
	{
		Integer C_Charge_ID = (Integer)value;
		if (C_Charge_ID == null || C_Charge_ID.intValue() == 0)
			return "";
		setCalloutActive(true);
		
		String sql = "SELECT IsTaxIncluded, ChargeAmt FROM C_Charge "
			+ "WHERE C_Charge_ID=? AND IsActive='Y' ";

		try
		{
			PreparedStatement pstmt = DB.prepareStatement(sql, null);
			pstmt.setInt(1, C_Charge_ID.intValue());
			ResultSet rs = pstmt.executeQuery();
			if (rs.next())
			{	
				BigDecimal chargeAmt = rs.getBigDecimal("ChargeAmt");
				if( chargeAmt != null && chargeAmt.floatValue()!=0.0f)
					mTab.setValue("Amount", chargeAmt);
				
				mTab.setValue("IsTaxIncluded", new Boolean("Y".equals(rs.getString("IsTaxIncluded"))));
			}
			rs.close();
			pstmt.close();
		}
		catch (SQLException e)
		{
			log.log(Level.SEVERE, sql, e);
			setCalloutActive(false);
			return e.getLocalizedMessage();
		}
		setCalloutActive(false);
		
		return tax(ctx, WindowNo, mTab, mField, value);
		
	}	//	charge
	
	/**
	 * @param ctx
	 * @param WindowNo
	 * @param mTab
	 * @param mField
	 * @param value
	 * @return
	 */
	public String tax (Ctx ctx, int WindowNo, GridTab mTab, GridField mField, Object value)
	{
		String cashType = (String)mTab.getValue("CashType");
		if( "C".equals(cashType) ){
			
			Integer C_Tax_ID = (Integer)mTab.getValue("C_Tax_ID");
			if( C_Tax_ID==null || C_Tax_ID == 0 )
				return "";
			
			setCalloutActive(true);

			MTax tax = MTax.get(ctx, C_Tax_ID);
			int C_Currency_ID = ctx.getContextAsInt( WindowNo, "C_Currency_ID");
			if( C_Currency_ID == 0 ){
				MClient client = MClient.get(ctx);
				C_Currency_ID = client.getC_Currency_ID();
			}
			int scale = MCurrency.getStdPrecision(ctx, C_Currency_ID);
			BigDecimal amount = (BigDecimal)mTab.getValue("Amount");
			if( amount == null )
				amount = BigDecimal.ZERO;
			
			boolean isTaxIncluded = true;
			Object obj = mTab.getValue("IsTaxIncluded");
			// if( obj instanceof String)
			//	isTaxIncluded = "Y".equals((String)obj);
			// else if( obj instanceof Boolean)
				isTaxIncluded = ((Boolean)obj).booleanValue();
			
			BigDecimal taxAmt = BigDecimal.ZERO;
			Integer C_BPartner_ID = (Integer)mTab.getValue("C_BPartner_ID");
			if( C_BPartner_ID!=null && C_BPartner_ID!=0 ) {
				MBPartner bpartner = MBPartner.get(ctx, C_BPartner_ID);		
				taxAmt = tax.calculateTax(amount, isTaxIncluded, scale, bpartner.getTaxRoundModeAsInt());
			} else {
				taxAmt = tax.calculateTax(amount, isTaxIncluded, scale, BigDecimal.ROUND_HALF_UP);
			}
			mTab.setValue("TaxAmt", taxAmt);
			if( !isTaxIncluded ){
				BigDecimal totalAmt = amount.add( taxAmt );
				mTab.setValue("TotalAmt", totalAmt);
			} else {
				mTab.setValue("TotalAmt", amount);
			}
			
			setCalloutActive(false);
			
		} else {
			
			setCalloutActive(true);

			BigDecimal amount = (BigDecimal)mTab.getValue("Amount");
			BigDecimal taxAmt = BigDecimal.ZERO;
			mTab.setValue("TaxAmt", taxAmt);
			mTab.setValue("TotalAmt", amount);
			
			setCalloutActive(false);
			
		}
		
		return "";
		
	}	// tax
	// Jirimuto modified for adding for charge tax calculation --end 2010/04/14
	
}	//	CalloutCashJournal
